import json
import logging
import os
import traceback
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError

from instances.models import Instance

logger = logging.getLogger(__name__)


def _stats_pretty(stats):
    return ' '.join(["%s:%d" % (cls.__name__, n) for cls, n in stats.items()])


class ImportCommand(BaseCommand):

    importer_class = None
    document_extension = ''

    option_list = BaseCommand.option_list + (
        make_option('--commit', action='store_true', help='Whether to commit to the database or not'),
        make_option('--instance', action='store', help='Label of instance to add data to'),
        make_option('--file', action='store', help='document to import'),
        make_option('--dir', action='store', help='directory of documents to import'),
        make_option(
            '--no-verify', action='store_false', default=True, dest='verify',
            help='Whether to verify SSL certificates or not'),
        make_option(
            '--start-date', action='store', default='',
            help='earliest date to process, in yyyy-mm-dd format'),
        make_option(
            '--dump-users', action='store', default='',
            help='dump a json list to <file> (only valid with --dir for now)'),
        make_option(
            '--clobber-existing', action='store_const', const='replace', dest='clobber',
            help='Whether to replace sections with the same heading'),
        make_option(
            '--skip-existing', action='store_const', const='skip', dest='clobber',
            help='Whether to skip sections with the same heading'),
        make_option(
            '--merge-existing', action='store_const', const='merge', dest='clobber',
            help='Whether to merge sections with the same heading'),
        )

    def handle(self, *args, **options):
        verbosity = int(options['verbosity'])

        if options['commit']:
            if not options['instance']:
                raise CommandError("You must specify an instance")
            instance, __ = Instance.objects.get_or_create(label=options['instance'])
        else:
            instance = Instance(label=options['instance'])
        options['instance'] = instance

        if options['file']:
            filename = os.path.expanduser(options['file'])
            (stats, speakers) = self.import_document(filename, **options)
            if verbosity > 1 and stats:
                logger.info("Imported %s\n\n" % _stats_pretty(stats))
        elif options['dir']:
            files = sorted(self.document_list(options))

            if len(files):
                speakers = {}
                for f in files:
                    (stats, spkrs) = self.import_document(f, **options)
                    speakers.update(spkrs)

                    if verbosity > 1 and stats:
                        logger.info("%s: Imported %s\n" % (f, _stats_pretty(stats)))

                dump_users = os.path.expanduser(options['dump_users'])
                if dump_users:
                    out = open(dump_users, 'w')
                    speakers_list = [(k, speakers[k]) for k in speakers]
                    out.write(json.dumps(speakers_list, indent=4))

                    if verbosity > 1:
                        logger.info("Saved speakers list to %s\n" % dump_users)
            else:
                logger.info("No .%s files found in directory" % self.document_extension)
        else:
            logger.info(self.help)

    def document_list(self, options):
        dir = os.path.expanduser(options['dir'])

        start_date = options['start_date']
        if start_date:
            def valid(f):
                return f >= start_date
        else:
            def valid(f):
                return True

        return [os.path.join(root, filename)
                for (root, _, files)
                in os.walk(dir)
                for filename in files
                if filename[-4:] == '.%s' % self.document_extension and valid(filename)]

    def document_valid(self, path):
        return os.path.isfile(path)

    def import_document(self, path, **options):
        verbosity = int(options['verbosity'])

        if not self.document_valid(path):
            raise CommandError("No document found")

        if verbosity > 1:
            logger.info("Starting import: %s\n" % path)

        if self.importer_class is None:
            raise CommandError("No importer_class specified!")

        importer = self.importer_class(**options)

        try:
            importer.import_document(path)
        except Exception as e:
            logger.error("An exception of type %s occurred, arguments:\n%s\n%s" % (
                type(e).__name__, e, traceback.format_exc()))
            return (None, {})

        return (importer.stats, importer.speakers)



from instances.models import Instance


class InstanceMiddleware:
    """This middleware sets request.instance to the default Instance for all
    requests. This can be changed/overridden if you use SayIt in a way that
    uses multiple instances."""
    def process_request(self, request):
        request.instance, _ = Instance.objects.get_or_create(label='default')
        request.is_user_instance = (
            hasattr(request, 'user') and request.user.is_authenticated() and
            (request.instance in request.user.instances.all() or request.user.is_superuser)
        )



from django.conf.urls import url, include
from django.views.decorators.csrf import csrf_exempt

from speeches.views import (
    AddAnSRedirectView, SpeechAudioCreate, SpeechCreate, SpeechUpdate,
    SpeechDelete, SpeechView, SpeakerCreate, SpeakerUpdate, SpeakerDelete,
    SpeakerView, SpeakerList, SectionCreate, SectionUpdate, SectionDelete,
    SectionView, SectionViewAN, ParentlessList, RecordingList, RecordingView,
    RecordingUpdate, RecordingAPICreate, InstanceView, Select2AutoResponseView,
    PopoloImportView, AkomaNtosoImportView,
    )

from speeches.search import InstanceSearchView

try:
    from tastypie.api import NamespacedApi
    from speeches.api import SpeechResource, SpeakerResource, SectionResource
    # XXX The below assumes this app is being included with a 'speeches' namespace.
    # Unclear how to have this inherit whichever namespace is being used.
    v01_api = NamespacedApi(api_name='v0.1', urlconf_namespace='speeches')
    v01_api.register(SpeakerResource())
    v01_api.register(SpeechResource())
    v01_api.register(SectionResource())
except:
    v01_api = None

urlpatterns = [
    url(r'^$', InstanceView.as_view(), name='home'),

    # Override the usual AutoResponseView from django_select2 so as to limit
    # results by instance
    url(r"^select2/fields/auto.json$",
        Select2AutoResponseView.as_view(),
        name="django_select2_central_json"),

    url(r'^(?P<path>speaker|recording)/?$', AddAnSRedirectView.as_view()),
    url(r'^(?P<path>speech)/?$', AddAnSRedirectView.as_view(suffix='es')),

    url(r'^search/', lambda request: InstanceSearchView()(request), name='haystack_search'),

    url(r'^speech/add$', SpeechCreate.as_view(), name='speech-add'),
    url(r'^speech/ajax_audio$', SpeechAudioCreate.as_view(), name='speech-ajax-audio'),
    url(r'^speech/(?P<pk>\d+)$', SpeechView.as_view(), name='speech-view'),
    url(r'^speech/(?P<pk>\d+)/edit$', SpeechUpdate.as_view(), name='speech-edit'),
    url(r'^speech/(?P<pk>\d+)/delete$', SpeechDelete.as_view(), name='speech-delete'),

    url(r'^speakers$', SpeakerList.as_view(), name='speaker-list'),
    url(r'^speaker/add$', SpeakerCreate.as_view(), name='speaker-add'),
    url(r'^speaker/(?P<pk>\d+)/edit$', SpeakerUpdate.as_view(), name='speaker-edit'),
    url(r'^speaker/(?P<pk>\d+)/delete$', SpeakerDelete.as_view(), name='speaker-delete'),
    url(r'^speaker/(?P<slug>.+)$', SpeakerView.as_view(), name='speaker-view'),

    url(r'^section/add$', SectionCreate.as_view(), name='section-add'),
    url(r'^section/(?P<pk>\d+)$', SectionView.as_view(), name='section-id-view'),
    url(r'^section/(?P<pk>\d+)/edit$', SectionUpdate.as_view(), name='section-edit'),
    url(r'^section/(?P<pk>\d+)/delete$', SectionDelete.as_view(), name='section-delete'),
    url(r'^speeches$', ParentlessList.as_view(), name='parentless-list'),

    url(r'^recordings$', RecordingList.as_view(), name='recording-list'),
    url(r'^recording/(?P<pk>\d+)$', RecordingView.as_view(), name='recording-view'),
    url(r'^recording/(?P<pk>\d+)/edit$', RecordingUpdate.as_view(), name='recording-edit'),
    url(r'^api/v0.1/recording/$', csrf_exempt(RecordingAPICreate.as_view()), name='recording-api-add'),

    url(r'^import/popolo', PopoloImportView.as_view(), name='import-popolo'),
    url(r'^import/akomantoso', AkomaNtosoImportView.as_view(), name='import-akoma-ntoso'),
]

if v01_api is not None:
    urlpatterns += [url(r'^api/', include(v01_api.urls))]

urlpatterns += [
    url(r'^(?P<full_slug>.+)\.an$', SectionViewAN.as_view(), name='section-view'),
    url(r'^(?P<full_slug>.+)$', SectionView.as_view(), name='section-view'),
]



import re
import sys

from django.conf import settings
from django.conf.urls.static import static
from django.conf.urls import include, url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.contrib.staticfiles import views as static_views
from django.contrib.auth import views as auth_views

# Admin section
from django.contrib import admin
admin.autodiscover()

urlpatterns = staticfiles_urlpatterns()
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# If we're running test, then we need to serve static files even though DEBUG
# is false to prevent lots of 404s. So do what staticfiles_urlpatterns would do.
if 'test' in sys.argv:
    static_url = re.escape(settings.STATIC_URL.lstrip('/'))
    urlpatterns += [
        url(r'^%s(?P<path>.*)$' % static_url, static_views.serve, {
            'insecure': True,
        }),
        url('^(?P<path>favicon\.ico)$', static_views.serve, {
            'insecure': True,
        }),
    ]

urlpatterns += [
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', include(admin.site.urls)),

    url(r'^accounts/login/$', auth_views.login, name='login'),
    url(r'^accounts/logout/$', auth_views.logout, {'next_page': '/'}, name='logout'),

    url(r'^', include('speeches.urls', app_name='speeches', namespace='speeches')),
]



from django.core.management.base import BaseCommand, CommandError

from instances.models import Instance
from speeches.importers.import_popolo import PopoloImporter


class Command(BaseCommand):
    args = '<base-url instance-label>'
    help = 'Imports people, memberships and organizations in Popolo format'

    def handle(self, *args, **options):
        if not args:
            raise CommandError('Please specify a source URL or file')

        base_url = args[0]

        if len(args) == 2:
            instance_label = args[1]
            try:
                instance = Instance.objects.get(label=instance_label)
            except Instance.DoesNotExist:
                raise CommandError(
                    'There is no instance with label %s' % instance_label)
        else:
            instance = None

        if len(args) > 2:
            raise CommandError('Too many arguments: %s' % repr(args))

        importer = PopoloImporter(base_url, instance=instance)
        importer.import_all()



import os.path
import json
import requests

from django.utils import six
from django.forms import ValidationError
from django.utils.translation import ugettext as _

from instances.models import Instance
from popolo.models import Membership, Organization, Post

from speeches.models import Speaker

import logging
logger = logging.getLogger(__name__)


def update_or_create(qs, defaults=None, **kwargs):
    defaults = defaults or {}
    obj, created = qs.get_or_create(defaults=defaults, **kwargs)

    if created:
        return obj, created
    for k, v in six.iteritems(defaults):
        setattr(obj, k, v)
    obj.save()
    return obj, False


def update_object(qs, data, extra=False, defaults=None, **kwargs):
    record, created = update_or_create(qs, defaults=defaults, **kwargs)

    for l in ('sources', 'links'):
        for item in data.get(l, []):
            rel = getattr(record, l)
            update_or_create(
                rel,
                # The following two rows were finally fixed to not be needed
                # in Django 1.7
                content_type=rel.content_type,
                object_id=rel.pk_val,
                url=item['url'],
                defaults={'note': item.get('note', '')}
            )
    # for link in data.get('contact_details', []):
    #     update_or_create(record.contact_details,
    #         #ContactDetail: label contact_type value note sources
    #     )
    if extra:
        update_or_create(
            record.identifiers,
            content_type=record.identifiers.content_type,
            object_id=record.identifiers.pk_val,
            identifier=data['id'],
            scheme='Popolo',
        )
        for i in data.get('identifiers', []):
            update_or_create(
                record.identifiers,
                content_type=record.identifiers.content_type,
                object_id=record.identifiers.pk_val,
                identifier=i['identifier'],
                scheme=i.get('scheme', ''),
            )
        for name in data.get('other_names', []):
            update_or_create(
                record.other_names,
                content_type=record.other_names.content_type,
                object_id=record.other_names.pk_val,
                name=name['name'],
                defaults={'note': name.get('note', '')},
            )

    return record, created


class PopoloImporterCreationError(ValidationError):
    pass


class PopoloImporter(object):
    popit_meta = None
    source_data = None

    def __init__(self, source, instance=None):
        if instance:
            self.instance = instance
        else:
            self.instance, created = Instance.objects.get_or_create(label='default')

        self.source = source

        try:
            if os.path.exists(self.source):
                json_src = json.load(open(self.source))
            elif self.source.startswith('http'):
                json_src = requests.get(source).json()
            else:
                raise PopoloImporterCreationError(
                    _('Either a file path or a URL is needed.'))
        except:
            raise PopoloImporterCreationError(
                _('Failed to decode JSON at %(source)s' % {'source': source}))

        # Look for meta and persons_api to identify popit.
        if type(json_src) == dict:
            # Look for persons key to identify source like
            # https://raw.githubusercontent.com/mysociety/pombola/0fd988606c31a31516ac782ebce00b9abfbb0c4d/pombola/south_africa/data/south-africa-popolo.json
            if 'persons' in json_src:
                self.source_data = json_src
                return

            try:
                json_src['meta']['persons_api_url']

                # This looks like sayit.
                self.popit_meta = json_src['meta']
                return
            except KeyError:
                pass

        elif type(json_src) is list:
            # Look for a single list, if we find one, assume it's
            # a list of persons
            self.source_data = {'persons': json_src}

        else:
            raise PopoloImporterCreationError(
                _('The json must contain either an object or an array'))

    def get_popit(self, path):
        data_url = self.popit_meta.get('%s_api_url' % path)

        while data_url:
            data = requests.get(data_url).json()

            if 'result' in data:
                data_url = data.get('next_url')

                # Looks like we have paginated data a la PopIt
                results = data['result']
            else:
                data_url = None

                # Probably we have all the data in a single file.
                results = data

            for x in results:
                yield x

    def get(self, path):
        if self.popit_meta:
            return self.get_popit(path)
        else:
            return self.source_data.get(path, [])

    def import_organizations(self):
        for data in self.get('organizations'):
            # Other fields that could be in defaults:
            # parent dissolution_date founding_date
            defaults = {
                # Limit to 128 characters as that's how big the name field
                # is in django-popolo
                'name': data['name'][:128],
                # 'classification': data['classification'],
            }
            update_object(
                Organization.objects, data, extra=True,
                identifiers__identifier=data['id'],
                defaults=defaults
            )

    def import_persons(self):
        created_count = 0
        refreshed_count = 0

        for data in self.get('persons'):
            # Other fields that could be in defaults:
            # additional_name honorific_prefix/suffix patronymic_name
            # gender birth_date death_date summary biography
            defaults = {
                'name': data['name'][:128],
                'family_name': data.get('family_name', ''),
                'given_name': data.get('given_name', ''),
                'sort_name': data.get('sort_name', ''),
                'email': data.get('email'),
                'image': data.get('image'),
            }
            record, created = update_object(
                Speaker.objects, data, extra=True,
                instance=self.instance, identifiers__identifier=data['id'],
                defaults=defaults
            )

            if created:
                created_count += 1
            else:
                refreshed_count += 1

        return {'created': created_count, 'refreshed': refreshed_count}

    def import_posts(self):
        self.posts = {}
        for data in self.get('posts'):
            defaults = {
                'role': data['role'],
                'organization': Organization.objects.get(
                    identifiers__identifier=data['organization_id']),
                'start_date': data.get('start_date', None),
                'end_date': data.get('end_date', None),
            }
            record, created = update_object(
                Post.objects, data,
                label=data['label'],
                defaults=defaults
            )
            if 'id' in data:
                self.posts[data['id']] = record

    def import_memberships(self):
        skipped_count = 0

        for data in self.get('memberships'):
            # Other fields that could be in defaults:
            # on_behalf_of
            defaults = {
                # @see https://github.com/openpolis/django-popolo/pull/12
                'label': data.get('label', ''),
                'role': data.get('role', ''),
                'start_date': data.get('start_date', None),
                'end_date': data.get('end_date', None),
            }

            if not data.get('organization_id'):
                # django-popolo doesn't yet cope with a membership
                # without an organization:
                # https://github.com/openpolis/django-popolo/pull/14
                logger.info(
                    'Skipping membership %s with no organization_id' %
                    data['id'])
                skipped_count += 1
                continue

            # XXX This uniqueness will fail for e.g. someone with multiple
            # memberships at the same organisation that don't have posts
            kwargs = {
                'organization': Organization.objects.get(
                    identifiers__identifier=data['organization_id']),
                'person': Speaker.objects.get(
                    identifiers__identifier=data['person_id']),
            }

            if data.get('post_id'):
                kwargs['post'] = self.posts[data['post_id']]

            update_object(
                Membership.objects, data,
                defaults=defaults, **kwargs
            )

        logger.info(
            'Skipped %d records for not having an organization' % skipped_count)

    def import_all(self):
        # Organizations, posts, and memberships in django-popolo are
        # not instance aware, and this really needs sorting out before
        # we start storing them. Fortunately, we only actually use
        # persons at the moment.

        # self.import_organizations()
        self.import_persons()
        # self.import_posts()
        # self.import_memberships()



import re
from datetime import datetime, date, time

from django.test import TestCase

from speeches.models import Section, Speech
from speeches.tests import create_sections
from instances.models import Instance
from speeches.tests import InstanceTestCase


class SectionModelTests(TestCase):

    def setUp(self):
        create_sections([
            {'heading': "Government Debates", 'subsections': [
                {'heading': "Monday 25th March", 'subsections': [
                    {'heading': "Oral Answers to Questions - Silly Walks",
                     'speeches': [4, date(2013, 3, 25), time(9, 0)],
                     },
                    {'heading': "Bill on Silly Walks",
                     'speeches': [2, date(2013, 3, 25), time(12, 0)],
                     },
                ]},
                {'heading': "Friday 29th March", 'subsections': [
                    {'heading': "Fixed Easter Bill", 'subsections': [
                        {'heading': "Z Clause",
                         'speeches': [2, date(2013, 3, 29), time(15, 00)],
                         },
                        {'heading': "Clause 1",
                         'speeches': [3, date(2013, 3, 29), time(14, 30)],
                         },
                        {'heading': "New Clause 1",
                         'speeches': [3, date(2013, 3, 29), time(14, 0)],
                         },
                    ]},
                ]},
            ]},
            {'heading': "Government Written Answers", 'subsections': [
                {'heading': "Ministry of Aardvarks", 'subsections': [
                    {'heading': "March",
                     'speeches': [3, None, None],
                     },
                ]},
                {'heading': "Ministry of Silly Walks", 'subsections': [
                    {'heading': "Wednesday 6th of March",
                     'speeches': [1, date(2013, 3, 6), None],
                     },
                    {'heading': "Thursday 7th of March",
                     'speeches': [2, date(2013, 3, 7), None],
                     },
                ]},
                {'heading': "Ministry of Something Else"
                 },
            ]},
        ])

    def test_datetimes(self):
        section = Section.objects.get(heading="Bill on Silly Walks")
        dts = sorted(section.speech_datetimes())
        self.assertEqual(dts[0], datetime(2013, 3, 25, 12, 0))
        self.assertEqual(dts[1], datetime(2013, 3, 25, 12, 10))

    def test_section_creation(self):

        # Get the top level section:
        top_level = Section.objects.get(heading='Government Written Answers')

        all_sections = top_level.get_descendants

        self.assertEqual(len(list(all_sections)),
                         6,
                         "The total number of sections was wrong")

        all_ministries = top_level.get_children

        self.assertEqual(len(all_ministries),
                         3,
                         "The number of ministries was wrong")

        # Check that the sections are in insertion order:
        all_ministries = [a.heading.replace('Ministry of ', '') for a in all_ministries]
        self.assertEqual(all_ministries, ['Aardvarks', 'Silly Walks', 'Something Else'])

        # Get all speeches under a section, where everything should be
        # sorted by speech date:
        top_level = Section.objects.get(heading="Government Debates")
        children = top_level.get_descendants
        children = [c.heading for c in children]
        self.assertEqual(children, [
            'Monday 25th March', 'Oral Answers to Questions - Silly Walks', 'Bill on Silly Walks',
            'Friday 29th March', 'Fixed Easter Bill', 'New Clause 1', 'Clause 1', 'Z Clause'])

    def test_section_next_previous(self):
        top_level = Section.objects.get(heading='Government Written Answers')
        depts = top_level.get_children
        self.assertEqual(depts[0].get_previous_node(), None)
        self.assertEqual(depts[1].get_previous_node(), depts[0])
        self.assertEqual(depts[2].get_previous_node(), depts[1])
        self.assertEqual(depts[0].get_next_node(), depts[1])
        self.assertEqual(depts[1].get_next_node(), depts[2])
        self.assertEqual(depts[2].get_next_node(), None)

        day = Section.objects.get(heading='Monday 25th March')
        e = Section.objects.get(heading='Fixed Easter Bill')
        debs = day.get_children
        self.assertEqual(debs[0].get_previous_node(), None)
        self.assertEqual(debs[1].get_previous_node(), debs[0])
        self.assertEqual(debs[0].get_next_node(), debs[1])
        self.assertEqual(debs[1].get_next_node(), e)

    def test_section_speech_next_previous(self):
        first_next = Section.objects.get(heading='Bill on Silly Walks').speech_set.all()[0]
        speeches = Section.objects.get(heading='Oral Answers to Questions - Silly Walks').speech_set.all()
        self.assertEqual(speeches[0].get_previous_speech(), None)
        self.assertEqual(speeches[0].get_next_speech(), speeches[1])
        self.assertEqual(speeches[1].get_next_speech(), speeches[2])
        self.assertEqual(speeches[2].get_next_speech(), speeches[3])
        self.assertEqual(speeches[3].get_next_speech(), first_next)

        speeches = Section.objects.get(heading='Ministry of Silly Walks') \
            .children.get(heading='Wednesday 6th of March').speech_set.all()
        p = Speech.objects.filter(section=Section.objects.get(heading='March')).reverse()[0]
        q = Speech.objects.filter(section=Section.objects.get(heading='Thursday 7th of March'))[0]
        self.assertEqual(speeches[0].get_previous_speech(), p)
        self.assertEqual(speeches[0].get_next_speech(), q)

    def test_section_descendant_speeches_queryset(self):
        top_level = Section.objects.get(heading='Government Written Answers')

        # Get all speeches
        speeches = top_level.descendant_speeches()
        self.assertEqual(speeches.count(), 6)

        # Check that filtering works as expected
        test_date = date(2013, 3, 7)
        on_test_date = speeches.filter(start_date=test_date)
        self.assertEqual(on_test_date.count(), 2)

    def test_section_get_or_create_with_parents(self):

        instance, _ = Instance.objects.get_or_create(label='get-or-create-with-parents')

        # Test that not passing an instance leads to an exception
        self.assertRaises(Exception, Section.objects.get_or_create_with_parents, {"headings": ("Foo", "Bar", "Baz")})

        # Create an initial set of sections
        baz_section = Section.objects.get_or_create_with_parents(instance=instance, headings=("Foo", "Bar", "Baz"))
        bar_section = baz_section.parent
        foo_section = bar_section.parent
        self.assertEqual(baz_section.heading, "Baz")
        self.assertEqual(baz_section.instance, instance)
        self.assertEqual(foo_section.parent, None)
        self.assertEqual(foo_section.instance, instance)

        # Create the same ones again, check same child section returned
        baz_again_section = Section.objects.get_or_create_with_parents(
            instance=instance, headings=("Foo", "Bar", "Baz"))
        self.assertEqual(baz_again_section, baz_section)

        # Create a similar set and check only new ones created
        bundy_section = Section.objects.get_or_create_with_parents(instance=instance, headings=("Foo", "Bar", "Bundy"))
        self.assertEqual(bundy_section.heading, "Bundy")
        self.assertEqual(bundy_section.parent, bar_section)


class SectionSiteTests(InstanceTestCase):
    """Tests for the section functionality"""

    def test_add_section_fails_on_empty_form(self):
        resp = self.client.post('/section/add')
        self.assertFormError(resp, 'form', None, 'You must specify at least one of num/heading/subheading')

    def test_add_section_with_heading(self):
        resp = self.client.post('/section/add', {
            'heading': 'A test section'
        })
        new_section = Section.objects.order_by('-id')[0]
        self.assertRedirects(resp, '%s' % new_section.slug)
        # Check in db
        section = Section.objects.get(id=new_section.id)
        self.assertEqual(section.heading, 'A test section')

    def test_add_section_in_section(self):
        section = Section.objects.create(heading='Test section', instance=self.instance)
        resp = self.client.post('/section/add', {
            'parent': section.id,
            'heading': 'A test subsection'
        })
        new_section = Section.objects.order_by('-id')[0]
        self.assertRedirects(resp, '%s/%s' % (section.slug, new_section.slug))
        # Check in db
        subsection = Section.objects.get(id=new_section.id)
        self.assertEqual(subsection.heading, 'A test subsection')
        self.assertEqual(subsection.parent, section)

    def test_section_page_lists_speeches(self):
        section = Section.objects.create(heading='A test section', instance=self.instance)
        subsection = Section.objects.create(heading='A test subsection', parent=section, instance=self.instance)

        # Assert no speeches
        resp = self.client.get('/section/%d' % subsection.id)
        self.assertSequenceEqual([], list(resp.context['section_tree']))

        speech = Speech.objects.create(
            text="A test speech",
            section=subsection,
            instance=self.instance,
            start_date=date(2014, 9, 17),
            )
        resp = self.client.get('/section/%d' % subsection.id)
        self.assertSequenceEqual(
            [(speech, {'speech': True, 'new_level': True, 'closed_levels': [1]})],
            list(resp.context['section_tree']))

        # Check that a second speech on the same date doesn't display a date
        Speech.objects.create(
            text="Second speech",
            section=subsection,
            instance=self.instance,
            start_date=date(2014, 9, 17),
            )
        Speech.objects.create(
            text="Next day speech",
            section=subsection,
            instance=self.instance,
            start_date=date(2014, 9, 18),
            )

        # Check that a section page which returns more than one speech on the
        # same date displays the date once.
        resp = self.client.get('/section/%d' % subsection.id)
        self.assertEqual(
            len(re.findall(
                r'<span class="speech__meta-data__date">\s*1[7,8] Sep 2014\s*</span>',
                resp.content.decode())),
            2)
        self.assertContains(resp, 'Next day speech')

    def test_section_page_lists_subsections(self):
        section = Section.objects.create(heading='A test section', instance=self.instance)

        # Assert no subsections
        resp = self.client.get('/section/%d' % section.id)
        self.assertSequenceEqual([], resp.context['section'].get_descendants)

        subsection = Section.objects.create(heading="A test subsection", parent=section, instance=self.instance)
        resp = self.client.get('/section/%d' % section.id)
        self.assertSequenceEqual([subsection], resp.context['section'].get_descendants)

    def test_section_page_has_buttons_to_edit(self):
        # Add a section
        section = Section.objects.create(heading='A test section', instance=self.instance)

        # Call the section's page
        resp = self.client.get('/section/%d' % section.id)

        self.assertContains(
            resp, '<a href="/speech/add?section=%d" class="button small right">Add speech</a>' % section.id,
            html=True)
        self.assertContains(
            resp,
            '<a href="/section/add?section=%d" class="button secondary small right">Add subsection</a>' % section.id,
            html=True)
        self.assertContains(
            resp, '<a href="/section/%d/edit" class="button secondary small right">Edit section</a>' % section.id,
            html=True)

    def test_section_deletion(self):
        # Set up the section
        section = Section.objects.create(heading='A test section', instance=self.instance)
        speech = Speech.objects.create(text="A test speech", section=section, instance=self.instance)
        resp = self.client.get('/section/%d' % section.id)
        self.assertSequenceEqual(
            [(speech, {'speech': True, 'new_level': True, 'closed_levels': [1]})],
            list(resp.context['section_tree']))

        # GET form (confirmation page)
        resp = self.client.get(section.get_delete_url())
        self.assertContains(resp, '<input type="submit" value="Confirm delete?"')

        section_db = Section.objects.get(id=section.id)
        self.assertEqual(section, section_db)

        speech_db = Speech.objects.get(id=speech.id)
        self.assertEqual(speech_db.section_id, section.id)

        # POST form (do the deletion)
        resp = self.client.post(section.get_delete_url())

        self.assertRedirects(resp, 'speeches')

        self.assertEqual(Section.objects.filter(id=section.id).count(), 0)

        speech_db = Speech.objects.get(id=speech.id)
        self.assertEqual(speech_db.section_id, None)

    def test_section_in_other_instance(self):
        other_instance = Instance.objects.create(label='other')
        section = Section.objects.create(heading='A test section', instance=other_instance)
        self.assertEqual(section.slug, 'a-test-section')
        section = Section.objects.create(heading='A test section', instance=self.instance)
        self.assertEqual(section.slug, 'a-test-section')
        section = Section.objects.create(heading='A test section', instance=other_instance)
        self.assertEqual(section.slug, 'a-test-section-2')
        resp = self.client.get('/a-test-section-2')
        self.assertContains(resp, 'Not Found', status_code=404)



import os
import tempfile
import shutil
import re
import datetime

from django.test.utils import override_settings
from django.conf import settings

from speeches.tests import InstanceTestCase

import speeches
from speeches.models import Speech, Speaker, Section

TEMP_MEDIA_ROOT = tempfile.mkdtemp()


@override_settings(MEDIA_ROOT=TEMP_MEDIA_ROOT)
class SpeechFormTests(InstanceTestCase):

    @classmethod
    def setUpClass(cls):
        cls._in_fixtures = os.path.join(os.path.abspath(speeches.__path__[0]), 'fixtures', 'test_inputs')
        super(SpeechFormTests, cls).setUpClass()

    def tearDown(self):
        # Clear the speeches folder if it exists
        speeches_folder = os.path.join(settings.MEDIA_ROOT, 'speeches')
        if(os.path.exists(speeches_folder)):
            shutil.rmtree(speeches_folder)

    def test_add_speech_page_exists(self):
        # Test that the page exists and has the right title
        resp = self.client.get('/speech/add')
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('Add a speech' in resp.content.decode())

    def test_add_speech_fails_on_empty_form(self):
        # Test that the form won't submit if empty
        resp = self.client.post('/speech/add')
        self.assertFormError(resp, 'form', None, 'You must provide either text or some audio')

    def test_add_speech_without_speaker(self):
        # Test form without speaker
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech'
        })
        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.text, 'This is a speech')
        self.assertEqual(speech.speaker, None)
        self.assertEqual(speech.type, 'narrative')

    def test_add_speech_with_unknown_speaker(self):
        """Try adding a speech with a speaker not yet in the database.

        Adding a speech with a speaker name which is unknown to us should cause
        that speaker to be created.
        """
        self.assertEqual(Speaker.objects.filter(name='New Speaker').count(), 0)
        # Note whitespace around speaker name to check it is stripped
        self.client.post(
            '/speech/add',
            {'text': 'Speech from new speaker',
             'speaker': ' New Speaker '},
            )

        speaker = Speaker.objects.get(name='New Speaker')
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(speech.text, '<p>Speech from new speaker</p>')
        self.assertEqual(speech.speaker, speaker)
        self.assertEqual(speech.type, 'speech')

    def test_add_speech_with_unknown_data_and_bad_form(self):
        """Try adding a speech with new speaker/section and no other data.

        What we want to happen in this case is for the new data
        to be created, and the form given back to the user to
        correct the other error with the new data displayed
        correctly.
        """

        self.assertEqual(Speaker.objects.filter(name='New Bod').count(), 0)
        self.assertEqual(Section.objects.filter(heading='New Section').count(), 0)

        resp = self.client.post('/speech/add', {'speaker': 'New Bod', 'section': 'New Section'})

        self.assertEqual(Speaker.objects.filter(name='New Bod').count(), 1)
        self.assertEqual(Section.objects.filter(heading='New Section').count(), 1)
        self.assertContains(resp, '.txt(["New Bod"])')
        self.assertContains(resp, '.txt(["New Section"])')
        self.assertContains(resp, "You must provide either text or some audio")

    def test_add_speech_with_unknown_section(self):
        """Try adding a speech with a section not yet in the database.

        Adding a speech with a section which is unknown to us should cause
        that section to be created.
        """
        self.assertEqual(Section.objects.filter(heading='New Section').count(), 0)
        self.client.post(
            '/speech/add',
            {'text': 'Speech in new section', 'section': 'New Section'},
        )

        section = Section.objects.get(heading='New Section')
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(speech.text, 'Speech in new section')
        self.assertEqual(speech.section, section)

    def test_add_speech_and_add_another(self):
        # Test form with 'add_another' but without a section
        # (client JS will prevent this case from happening, in general)
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech',
            'add_another': 1,
        })
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(speech.text, 'This is a speech')
        self.assertRedirects(resp, '/speech/add')

        section = Section.objects.create(heading='Test', instance=self.instance)
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech',
            'section': section.id,
            'add_another': 1,
        })
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(speech.text, 'This is a speech')
        self.assertEqual(speech.section_id, section.id)
        get_url = '/speech/add?section=%d&added=%d' % (section.id, speech.id)
        self.assertRedirects(resp, get_url)

        resp = self.client.get(get_url)
        self.assertContains(resp, 'Your speech has been <a href="/speech/%d">created</a>' % speech.id)
        self.assertContains(resp, 'in the section <a href="/%s#s%d">%s</a>!' % (
            section.slug, speech.id, section.heading))

    def _post_speech(self, section, speech_data):
        post_data = speech_data.copy()
        try:
            post_data['start_date'] = speech_data['start_date'].strftime('%d/%m/%Y')
            post_data['end_date'] = speech_data['end_date'].strftime('%d/%m/%Y')
        except KeyError:
            pass
        resp = self.client.post('/speech/add', post_data)
        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/add?section=%d&added=%d' % (section.id, speech.id))
        return resp

    def _check_initial_speech_data(self, response, section, speaker, speech_data):
        initial = response.context['form'].initial

        self.assertTrue('text' not in initial)

        self.assertEqual(initial['section'], section)
        self.assertEqual(initial['speaker'], speaker)
        for x in ['heading', 'event', 'location', 'public', 'source_url', 'start_date', 'start_time']:
            self.assertEqual(initial.get(x, None), speech_data.get(x, None))

    def test_add_speech_metadata_in_section(self):
        section = Section.objects.create(heading='Test', instance=self.instance)

        speakers = [Speaker.objects.create(name='Speaker %d' % i, instance=self.instance) for i in range(3)]

        speech_data = {
            'text': 'This is a speech',
            'section': section.id,
            'heading': 'Title',
            'event': 'Event',
            'location': 'Location',
            'speaker': speakers[0].id,
            'public': True,
            'source_url': 'http://example.com/speeches/1',
            'start_date': datetime.date(year=2000, month=1, day=15),
            'start_time': datetime.time(hour=12, minute=0, second=0),
            'add_another': 1,
        }

        form_url = '/speech/add?section=%d' % section.id

        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        speech_data['start_time'] = datetime.time(hour=12, minute=0, second=10)
        self._check_initial_speech_data(resp, section, speakers[0], speech_data)

        speech_data['speaker'] = speakers[1].id
        speech_data['end_date'] = speech_data['start_date']
        speech_data['end_time'] = datetime.time(hour=12, minute=0, second=30)
        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        speech_data['start_time'] = datetime.time(hour=12, minute=0, second=30)
        self._check_initial_speech_data(resp, section, speakers[0], speech_data)

        speech_data['speaker'] = speakers[2].id
        speech_data['start_time'] = datetime.time(hour=12, minute=0, second=30)
        speech_data['end_date'] = speech_data['start_date']
        speech_data['end_time'] = datetime.time(hour=12, minute=0, second=30)
        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)

        speech_data['start_time'] = datetime.time(hour=12, minute=0, second=31)
        self._check_initial_speech_data(resp, section, speakers[1], speech_data)

        # Test without start/end times
        del speech_data['start_date']
        del speech_data['start_time']
        del speech_data['end_date']
        del speech_data['end_time']
        speech_data['speaker'] = speakers[1].id
        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        self._check_initial_speech_data(resp, section, speakers[2], speech_data)

        # TODO: also default Tags

    def test_add_speech_metadata_start_date_but_no_time(self):
        section = Section.objects.create(heading='Test', instance=self.instance)

        speakers = [Speaker.objects.create(name='Speaker %d' % i, instance=self.instance) for i in range(3)]

        speech_data = {
            'text': 'This is a speech',
            'section': section.id,
            'heading': 'Title',
            'event': 'Event',
            'location': 'Location',
            'speaker': speakers[0].id,
            'public': True,
            'source_url': 'http://example.com/speeches/1',
            'start_date': datetime.date(year=2000, month=1, day=15),
            'add_another': 1,
        }

        form_url = '/speech/add?section=%d' % section.id

        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        self._check_initial_speech_data(resp, section, speakers[0], speech_data)

        speech_data['speaker'] = speakers[1].id
        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        self._check_initial_speech_data(resp, section, speakers[0], speech_data)

        speech_data['speaker'] = speakers[0].id
        resp = self._post_speech(section, speech_data)

        resp = self.client.get(form_url)
        self._check_initial_speech_data(resp, section, speakers[1], speech_data)

    def test_add_speech_with_speaker(self):
        # Test form with speaker, we need to add a speaker first
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)

        resp = self.client.post('/speech/add', {
            'text': 'This is a Steve speech',
            'speaker': speaker.id
        })
        # Check in db
        speech = Speech.objects.get(speaker=speaker.id)
        self.assertEqual(speech.text, '<p>This is a Steve speech</p>')
        self.assertEqual(speech.type, 'speech')

        # Check that the edit page for this speech contains the speaker's name
        resp = self.client.get('/speech/{}/edit'.format(speech.id))
        self.assertContains(resp, '.txt(["Steve"])')

    def test_add_and_remove_speaker_from_speech(self):
        # Test form with speaker, we need to add a speaker first
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)

        orig_text = 'This is a Steve speech\nAfter break\n\nNew paragraph'
        with_speaker_text = '<p>This is a Steve speech<br />After break</p>\n\n<p>New paragraph</p>'
        no_speaker_text = 'This is a Steve speech<br />After break<br />\nNew paragraph'

        resp = self.client.post('/speech/add', {
            'text': orig_text,
            'speaker': speaker.id,
        })

        speech = Speech.objects.get(speaker_id=speaker.id)
        self.assertEqual(speech.text, with_speaker_text)
        self.assertEqual(speech.type, 'speech')

        resp = self.client.get('/speech/{}/edit'.format(speech.id))
        self.assertTrue(orig_text in resp.content.decode())

        self.client.post(
            '/speech/{}/edit'.format(speech.id),
            {'text': orig_text},
            )

        speech = Speech.objects.get(id=speech.id)
        self.assertEqual(speech.text, no_speaker_text)
        self.assertEqual(speech.type, 'narrative')

        resp = self.client.get('/speech/{}/edit'.format(speech.id))
        self.assertTrue(orig_text in resp.content.decode())

        self.client.post(
            '/speech/{}/edit'.format(speech.id),
            {'text': orig_text, 'speaker': speaker.id},
            )

        speech = Speech.objects.get(speaker_id=speaker.id)
        self.assertEqual(speech.text, with_speaker_text)
        self.assertEqual(speech.type, 'speech')

    def test_add_speech_with_audio(self):
        # Load the mp3 fixture
        audio = open(os.path.join(self._in_fixtures, 'lamb.mp3'), 'rb')

        resp = self.client.post('/speech/add', {
            'audio': audio
        })
        # Assert that it uploads and we're told to wait
        speech = Speech.objects.order_by('-id')[0]
        resp = self.client.get('/speech/%d' % speech.id)
        self.assertContains(resp, 'recorded audio')

        # Assert that it's in the model
        self.assertIsNotNone(speech.audio)

    def test_add_speech_with_audio_and_text(self):
        # Load the mp3 fixture
        audio = open(os.path.join(self._in_fixtures, 'lamb.mp3'), 'rb')
        text = 'This is a speech with some text'

        resp = self.client.post('/speech/add', {
            'audio': audio,
            'text': text
        })

        # Assert that it uploads and we see it straightaway
        speech = Speech.objects.order_by('-id')[0]
        resp = self.client.get('/speech/%d' % speech.id)
        self.assertFalse('Please wait' in resp.content.decode('utf-8'))
        self.assertTrue(text in resp.content.decode('utf-8'))

        # Test edit page
        resp = self.client.get('/speech/%d/edit' % speech.id)

    def test_add_speech_with_whitespace_around_text(self):
        text = ' This is a speech with whitespace at the ends. '
        self.client.post('/speech/add', {'text': text})

        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(speech.text, 'This is a speech with whitespace at the ends.')

    def test_add_speech_with_newlines(self):
        text = "First line.\nAfter break.\n\nAfter another break."
        self.client.post('/speech/add', {'text': text})
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(
            speech.text,
            'First line.<br />After break.<br />\nAfter another break.'
            )

        resp = self.client.get("/speech/%d/edit" % speech.id)
        self.assertContains(resp, text)

    def test_add_speech_with_newlines_and_speaker(self):
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)
        text = "First line.\nAfter break.\n\nNew paragraph."
        self.client.post(
            '/speech/add',
            {'text': text, 'speaker': speaker.id},
            )
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(
            speech.text,
            '<p>First line.<br />After break.</p>\n\n<p>New paragraph.</p>'
            )

        resp = self.client.get("/speech/%d/edit" % speech.id)
        self.assertContains(resp, text)

    def test_add_speech_with_html(self):
        """If a user adds a speech which starts with a <p>, we're going to have
        to take those out, as Akoma Ntoso doesn't allow them in <narrative> elemnts.

        What we do want to do though is make sure that every </p> is followed by two
        newlines (\n) so that when we take the <p>s etc out to edit we can see where
        to put them back in again.
        """

        text = "<p>Test<br />string</p><p>Second paragraph</p>\n<p>Third paragraph</p>\n\n<p>Fourth paragraph</p>"
        self.client.post('/speech/add', {'text': text})
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(
            speech.text,
            "Test<br />string<br />\nSecond paragraph<br />\nThird paragraph<br />\nFourth paragraph"
            )

    def test_add_speech_with_html_and_speaker(self):
        """If a user adds a speech which starts with a <p>, we probably
        don't want to add more <p>s to it.

        What we do want to do though is make sure that every </p> is followed by two
        newlines (\n) so that when we take the <p>s etc out to edit we can see where
        to put them back in again.
        """
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)
        text = "<p>Test<br />string</p><p>Second paragraph</p>\n<p>Third paragraph</p>\n\n<p>Fourth paragraph</p>"
        self.client.post('/speech/add', {'text': text, 'speaker': speaker.id})
        speech = Speech.objects.order_by('-id')[0]
        self.assertEqual(
            speech.text,
            "<p>Test<br />string</p>\n\n<p>Second paragraph</p>\n\n<p>Third paragraph</p>\n\n<p>Fourth paragraph</p>"
            )

    def test_add_speech_fails_with_unsupported_audio(self):
        # Load the .aiff fixture
        audio = open(os.path.join(self._in_fixtures, 'lamb.aiff'), 'rb')

        resp = self.client.post('/speech/add', {
            'audio': audio
        })

        # Assert that it fails and gives us an error
        self.assertFormError(resp, 'form', 'audio', 'That file does not appear to be an audio file')

    def test_add_speech_with_dates_only(self):
        # Test form with dates (but not times)
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech',
            'start_date': '01/01/2000',
            'end_date': '01/01/2000'
        })
        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.start_date, datetime.date(year=2000, month=1, day=1))
        self.assertIsNone(speech.start_time)
        self.assertEqual(speech.end_date, datetime.date(year=2000, month=1, day=1))
        self.assertIsNone(speech.end_time)

    def test_add_speech_form_internationalization(self):
        en_gb_language_headers = {'HTTP_ACCEPT_LANGUAGE': 'en-GB'}
        en_language_headers = {'HTTP_ACCEPT_LANGUAGE': 'en'}

        en_gb_resp = self.client.get('/speech/add', **en_gb_language_headers)
        self.assertContains(en_gb_resp, 'placeholder="dd/mm/yyyy"')

        resp = self.client.post(
            '/speech/add',
            {'text': 'This is a speech',
             'start_date': '20/01/2000',
             'end_date': '21/01/2000'},
            **en_gb_language_headers)

        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.start_date, datetime.date(year=2000, month=1, day=20))
        self.assertEqual(speech.end_date, datetime.date(year=2000, month=1, day=21))

        en_resp = self.client.get('/speech/add', **en_language_headers)
        self.assertContains(en_resp, 'placeholder="yyyy-mm-dd"')

        # With language 'en', the default format is ISO8601
        resp = self.client.post(
            '/speech/add',
            {'text': 'This is a speech',
             'start_date': '2000-01-20',
             'end_date': '2000-01-21'},
            **en_language_headers)

        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.start_date, datetime.date(year=2000, month=1, day=20))
        self.assertEqual(speech.end_date, datetime.date(year=2000, month=1, day=21))

        # With language 'en', django will also handle dates in the US back to
        # front fashion.
        resp = self.client.post(
            '/speech/add',
            {'text': 'This is a speech',
             'start_date': '01/20/2000',
             'end_date': '01/21/2000'},
            **en_language_headers)

        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.start_date, datetime.date(year=2000, month=1, day=20))
        self.assertEqual(speech.end_date, datetime.date(year=2000, month=1, day=21))

    def test_add_speech_with_dates_and_times(self):
        # Test form with dates (but not times)
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech',
            'start_date': '01/01/2000',
            'start_time': '12:45',
            'end_date': '01/01/2000',
            'end_time': '17:53'
        })
        speech = Speech.objects.order_by('-id')[0]
        self.assertRedirects(resp, '/speech/%d?created' % speech.id)
        self.assertEqual(speech.start_date, datetime.date(year=2000, month=1, day=1))
        self.assertEqual(speech.start_time, datetime.time(hour=12, minute=45))
        self.assertEqual(speech.end_date, datetime.date(year=2000, month=1, day=1))
        self.assertEqual(speech.end_time, datetime.time(hour=17, minute=53))

    def test_add_speech_fails_with_times_only(self):
        # Test form with dates (but not times)
        resp = self.client.post('/speech/add', {
            'text': 'This is a speech',
            'start_time': '12:45',
            'end_time': '17:53'
        })
        self.assertFormError(resp, 'form', 'start_time', 'If you provide a start time you must give a start date too')
        self.assertFormError(resp, 'form', 'end_time', 'If you provide an end time you must give an end date too')

    def test_add_speech_with_audio_encodes_to_mp3(self):
        # Load the wav fixture
        audio = open(os.path.join(self._in_fixtures, 'lamb_stereo.wav'), 'rb')

        resp = self.client.post('/speech/add', {
            'audio': audio
        })

        # Assert that it uploads
        speech = Speech.objects.order_by('-id')[0]
        resp = self.client.get('/speech/%d' % speech.id)
        self.assertContains(resp, 'recorded audio')

        self.assertIsNotNone(speech.audio)
        self.assertTrue(".mp3" in speech.audio.path)


class SpeechViewTests(InstanceTestCase):
    def test_speech_heading_display(self):
        speech = Speech.objects.create(
            text='Speech text',
            heading='Speech title',
            instance=self.instance,
            start_date=datetime.date(2014, 9, 17),
            )
        resp = self.client.get('/speech/%d' % speech.id)
        self.assertContains(resp, '<h1>Speech title</h1>')
        resp = self.client.get('/speeches')
        self.assertContains(resp, 'Speech title')

        # Check that two speeches on the same date both have their date displayed
        Speech.objects.create(
            text='Another speech',
            instance=self.instance,
            start_date=datetime.date(2014, 9, 17),
            )

        resp = self.client.get('/speeches')
        assert len(re.findall(
            r'<span class="speech__meta-data__date">\s*17 Sep 2014\s*</span>',
            resp.content.decode())) == 2

        section = Section.objects.create(heading='Test', instance=self.instance)
        speech.section = section
        speech.save()
        resp = self.client.get('/section/%d' % section.id)
        self.assertContains(resp, 'Speech title')

    def test_visible_speeches(self):
        section = Section.objects.create(heading='Test', instance=self.instance)
        speeches = []
        for i in range(3):
            s = Speech.objects.create(text='Speech %d' % i, section=section, instance=self.instance, public=(i == 2))
            speeches.append(s)

        resp = self.client.get('/section/%d' % section.id)
        self.assertEqual([x[0].public for x in resp.context['section_tree']], [False, False, True])
        self.assertContains(resp, 'Invisible', count=2)

        self.client.logout()
        resp = self.client.get('/speech/%d' % speeches[2].id)
        self.assertContains(resp, 'Speech 2')
        resp = self.client.get('/speech/%d' % speeches[0].id)
        self.assertContains(resp, 'Not Found', status_code=404)

    def test_speech_datetime_line(self):
        section = Section.objects.create(heading='Test', instance=self.instance)
        Speech.objects.create(
            text='Speech', section=section, instance=self.instance,
            public=True, start_date=datetime.date(2000, 1, 1), end_date=datetime.date(2000, 1, 2)
        )

        resp = self.client.get('/section/%d' % section.id)
        self.assertRegexpMatches(resp.content.decode(), '>\s+1 Jan 2000\s+&ndash;\s+2 Jan 2000\s+<')

    def test_speech_page_has_buttons_to_edit(self):
        # Add a section
        speech = Speech.objects.create(text="A test speech", instance=self.instance)

        # Call the speech's page
        resp = self.client.get('/speech/%d' % speech.id)

        self.assertContains(
            resp, '<a href="/speech/%d/edit"><i class="speech-icon icon-edit"></i>Edit</a>' % speech.id, html=True)
        self.assertContains(
            resp, '<a href="/speech/%d/delete"><i class="speech-icon icon-delete"></i>Delete</a>' % speech.id,
            html=True)

    def test_speech_deletion(self):
        # Set up the section
        speech = Speech.objects.create(text="A test speech", instance=self.instance)
        resp = self.client.get('/speech/%d' % speech.id)

        # GET form (confirmation page)
        resp = self.client.get(speech.get_delete_url())
        self.assertContains(resp, '<input type="submit" value="Confirm delete?"')

        speech_db = Speech.objects.get(id=speech.id)
        self.assertEqual(speech_db.id, speech.id)

        # POST form (do the deletion)
        resp = self.client.post(speech.get_delete_url())

        self.assertRedirects(resp, 'speeches')

        self.assertEqual(Speech.objects.filter(id=speech.id).count(), 0)


class SpeechTests(InstanceTestCase):
    def test_speech_with_i18n(self):
        speaker = Speaker.objects.create(name=u'Beyonc\u00e9', instance=self.instance)
        speech = Speech.objects.create(
            heading=u'D\u00e9j\u00e0 Vu', text='Bass', speaker=speaker, instance=self.instance)
        self.assertEqual(u'%s' % speech, u'Speech, D\u00e9j\u00e0 Vu by Beyonc\u00e9 (with text)')



import datetime
from itertools import chain

from django import forms
from django.core import validators
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.forms.utils import flatatt

from django_select2.widgets import Select2MultipleWidget

from speeches.models import Tag


# prepare_value() is called by a form before sending to the widget for display.
# clean() is called on form submission to validate it.
class FromStartIntegerField(forms.IntegerField):
    def __init__(self, *args, **kwargs):
        kwargs['min_value'] = 0
        super(FromStartIntegerField, self).__init__(*args, **kwargs)

    def prepare_value(self, value):
        if value in validators.EMPTY_VALUES:
            return None
        if isinstance(value, datetime.datetime):
            value = value - self.recording_start
            value = value.seconds + value.days * 86400
            value = int(value)
        return value

    def clean(self, value):
        value = super(FromStartIntegerField, self).clean(value)
        if value in validators.EMPTY_VALUES:
            return None
        # Get the number of seconds back to a datetime
        return datetime.timedelta(seconds=value) + self.recording_start


# This is the unfinished start of getting a Select2 tags:... element to work,
# so that you can add new tags within an edit form and have the server add them
# automatically. The client side below works, displaying the tags currently
# associated and allowing editing/adding, but server submission does not yet
# work. TagField needs to add any tags that aren't already present in the
# database.

class TagWidget(forms.SelectMultiple):
    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = []
        final_attrs = self.build_attrs(attrs, type='text', name=name)

        value = set(force_text(v) for v in value)
        selected = []
        all_tags = []
        for option_value, option_label in chain(self.choices, choices):
            option_value = force_text(option_value)
            option_label = force_text(option_label)
            if option_value in value:
                selected.append(option_label)
            all_tags.append(option_label)

        self.options['tags'] = all_tags
        self.options['width'] = 'resolve'

        if selected:
            final_attrs['value'] = ','.join(selected)
        return mark_safe(u'<input%s />' % flatatt(final_attrs))


class TagWidgetMixin(Select2MultipleWidget, TagWidget):
    pass


class TagField(forms.ModelMultipleChoiceField):
    widget = TagWidgetMixin

    def __init__(self, *args, **kwargs):
        kwargs['required'] = False
        super(TagField, self).__init__(Tag, *args, **kwargs)

    def clean(self, value):
        # if value:
        #     value = [ item.strip() for item in value.split(",") ]
        return super(TagField, self).clean(value)



import lxml.etree as etree
from speeches.external.formencode import xml_compare

from speeches.tests import InstanceTestCase
from speeches.models import Speech, Section, Speaker


class AkomaNtosoOutputTestCase(InstanceTestCase):
    def test_output_speech(self):
        speaker = Speaker.objects.create(
            instance=self.instance,
            name='Test Speaker',
            )
        section = Section.objects.create(
            instance=self.instance,
            heading='Test Section')
        Speech.objects.create(
            instance=self.instance,
            text='Test Speech',
            speaker=speaker,
            type='speech',
            section=section,
            )

        resp = self.client.get('/test-section.an')
        output = resp.content
        lxml1 = etree.fromstring(output)

        expected = """
            <akomaNtoso>
              <debate>
                <meta>
                  <references>
                    <TLCPerson href="/ontology/person/testserver/test-speaker"
                               id="test-speaker" showAs="Test Speaker"/>
                  </references>
                </meta>
                <debateBody>
                  <debateSection>
                    <heading>Test Section</heading>
                    <speech by="#test-speaker">
                      Test Speech
                    </speech>
                  </debateSection>
                </debateBody>
              </debate>
            </akomaNtoso>
            """
        lxml2 = etree.fromstring(expected)

        assert xml_compare(lxml1, lxml2)

    def test_output_scene(self):
        section = Section.objects.create(
            instance=self.instance,
            heading='Test Section')
        Speech.objects.create(
            instance=self.instance,
            text='Test Scene',
            type='scene',
            section=section,
            )

        resp = self.client.get('/test-section.an')
        output = resp.content
        lxml1 = etree.fromstring(output)

        expected = """
            <akomaNtoso>
              <debate>
                <meta>
                  <references>
                  </references>
                </meta>
                <debateBody>
                  <debateSection>
                    <heading>Test Section</heading>
                    <scene>
                      Test Scene
                    </scene>
                  </debateSection>
                </debateBody>
              </debate>
            </akomaNtoso>
            """
        lxml2 = etree.fromstring(expected)

        assert xml_compare(lxml1, lxml2)

    def test_random_speech_type_renders_as_other(self):
        section = Section.objects.create(
            instance=self.instance,
            heading='Test Section')
        Speech.objects.create(
            instance=self.instance,
            text='Text here',
            type='random',
            section=section,
            )

        resp = self.client.get('/test-section.an')
        output = resp.content
        lxml1 = etree.fromstring(output)

        expected = """
            <akomaNtoso>
              <debate>
                <meta>
                  <references>
                  </references>
                </meta>
                <debateBody>
                  <debateSection>
                    <heading>Test Section</heading>
                    <other>
                      Text here
                    </other>
                  </debateSection>
                </debateBody>
              </debate>
            </akomaNtoso>
            """
        lxml2 = etree.fromstring(expected)

        assert xml_compare(lxml1, lxml2)

    def test_empty_section(self):
        Section.objects.create(
            instance=self.instance,
            heading='Test Section')

        resp = self.client.get('/test-section.an')
        output = resp.content
        lxml1 = etree.fromstring(output)

        expected = """
            <akomaNtoso>
              <debate>
                <meta>
                  <references>
                  </references>
                </meta>
                <debateBody>
                  <debateSection>
                    <heading>Test Section</heading>
                  </debateSection>
                </debateBody>
              </debate>
            </akomaNtoso>
            """
        lxml2 = etree.fromstring(expected)

        assert xml_compare(lxml1, lxml2)

    def test_section_containing_empty_section(self):
        section = Section.objects.create(
            instance=self.instance,
            heading='Outer Section')

        Section.objects.create(
            instance=self.instance,
            heading='Inner Section',
            parent=section)

        resp = self.client.get('/outer-section.an')
        output = resp.content
        lxml1 = etree.fromstring(output)

        expected = """
            <akomaNtoso>
              <debate>
                <meta>
                  <references>
                  </references>
                </meta>
                <debateBody>
                  <debateSection>
                    <heading>Outer Section</heading>
                    <debateSection>
                      <heading>Inner Section</heading>
                    </debateSection>
                  </debateSection>
                </debateBody>
              </debate>
            </akomaNtoso>
            """
        lxml2 = etree.fromstring(expected)

        assert xml_compare(lxml1, lxml2)

    def test_nested_sections_tree(self):
        section = Section.objects.create(
            instance=self.instance, heading='Outer Section')
        inner1 = Section.objects.create(
            instance=self.instance, heading='Inner 1', parent=section)
        inner2 = Section.objects.create(
            instance=self.instance, heading='Inner 2', parent=section)

        resp = self.client.get('/outer-section.an')
        self.assertSequenceEqual([
            (inner1, {'new_level': True}),
            (inner2, {'closed_levels': [1]}),
            ], list(resp.context['section_tree']))

        speech1 = Speech.objects.create(
            text="A test speech", section=inner1, instance=self.instance)
        speech2 = Speech.objects.create(
            text="A test speech", section=inner2, instance=self.instance)

        resp = self.client.get('/outer-section.an')
        self.assertSequenceEqual([
            (inner1, {'new_level': True}),
            (speech1,
                {'closed_levels': [2], 'new_level': True, 'speech': True}),
            (inner2, {}),
            (speech2,
                {'closed_levels': [2, 1], 'new_level': True, 'speech': True}),
            ], list(resp.context['section_tree']))



from django.utils.html import escape, conditional_escape
from django.utils.safestring import mark_safe
from django.utils import formats
from django.utils.translation import ugettext_lazy as _

from django.forms.widgets import ClearableFileInput, CheckboxInput, DateInput, TimeInput


class AudioFileInput(ClearableFileInput):
    pretty_input_start = u'<span class="button tiny secondary fileinput-button">Choose audio file'
    pretty_input_end = u'</span>'
    template_with_initial = u'%(initial_text)s: %(initial)s %(clear_template)s<br />' \
        u'%(input_text)s: %(pretty_input_start)s%(input)s%(pretty_input_end)s'
    template_with_clear = u'%(clear)s <label class="inline checkbox" for="%(clear_checkbox_id)s">' \
        u'%(clear_checkbox_label)s</label>'

    # Overridden whole function to provide nicer input button
    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
            'pretty_input_start': self.pretty_input_start,
            'pretty_input_end': self.pretty_input_end,
        }
        template = u'%(pretty_input_start)s%(input)s%(pretty_input_end)s'
        substitutions['input'] = super(ClearableFileInput, self).render(name, value, attrs)

        if value and hasattr(value, "url"):
            template = self.template_with_initial
            substitutions['initial'] = (u'<audio src="%s" controls></audio>'
                                        % (escape(value.url),))
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions['clear_checkbox_name'] = conditional_escape(checkbox_name)
                substitutions['clear_checkbox_id'] = conditional_escape(checkbox_id)
                substitutions['clear'] = CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
                substitutions['clear_template'] = self.template_with_clear % substitutions

        return mark_safe(template % substitutions)


# Make sure likely date placeholder texts are in the translation strings file.
_('yyyy-mm-dd')
_('dd/mm/yyyy')
_('dd/mm/yy')
_('mm/dd/yyyy')
_('mm/dd/yy')


class DatePickerWidget(DateInput):
    """A widget for replacing text input for dates with a date picker.

    An input with type=text is the default for a date in Django, and looks
    set to continue being for a while:
    https://code.djangoproject.com/ticket/16630#comment:11

    This widget replaces this with a foundation datepicker - details at
    http://foundation-datepicker.peterbeno.com/example.html
    """

    def render(self, name, value, attrs=None):
        """Extend the output to return a foundation-datepicker."""

        # Set a placeholder attribute - this should be the right thing
        # based on the locale which goes with your browser language
        # setting - so 'dd/mm/yyyy' for the UK
        date_format = (
            formats.get_format('DATE_INPUT_FORMATS')[0]
            .replace('%d', 'dd')
            .replace('%m', 'mm')
            .replace('%y', 'yy')
            .replace('%Y', 'yyyy')
            )

        attrs['placeholder'] = _(date_format)
        # We'll base the format for the date picker on a separate attribute
        # so that the placeholder can be translated.
        attrs['datepicker-format'] = date_format

        # Add a class attribute so that we can generically javascript things
        attrs['class'] = (attrs.get('class', '') + " fdatepicker").strip()

        return mark_safe(
            u'<div class="input-append">' +
            super(DatePickerWidget, self).render(name, value, attrs) +
            u'</div>'
            )


class TimePickerWidget(TimeInput):
    """A Widget for replacing text input for times.

    An input with type=text is the default for a time in Django, and looks
    set to continue being for a while:
    https://code.djangoproject.com/ticket/16630#comment:11

    This widget currently just sets a placeholder, but should probably
    replace this with a time picker of some sort.
    """

    def render(self, name, value, attrs=None):
        """Extend the output to include a placeholder."""

        # Set a placeholder attribute
        attrs['placeholder'] = _('hh:mm')

        return super(TimePickerWidget, self).render(name, value, attrs)



from speeches.tests import InstanceTestCase
from speeches.models import Speech, Speaker, Section


class SmokeTests(InstanceTestCase):
    """Simple smoke tests (is it up?) of all the urls on the site"""

    def test_home_page(self):
        resp = self.client.get('/')
        self.assertEqual(resp.status_code, 200)

    def test_all_speeches_page(self):
        resp = self.client.get('/speeches')
        self.assertEqual(resp.status_code, 200)

    def test_a_speaker_page(self):
        # Add a speaker first
        speaker = Speaker.objects.create(name='Steve', instance=self.instance)
        resp = self.client.get('/speaker/%s' % speaker.slug)
        self.assertEqual(resp.status_code, 200)

    def test_a_speech_page(self):
        # Add a speech first
        speech = Speech.objects.create(text='Testing speech page', instance=self.instance)
        resp = self.client.get('/speech/%s' % speech.id)
        self.assertEqual(resp.status_code, 200)

    def test_add_speech_page(self):
        resp = self.client.get('/speech/add')
        self.assertEqual(resp.status_code, 200)

    def test_a_section_page(self):
        # Add a section first
        section = Section.objects.create(heading="A Section", instance=self.instance)
        resp = self.client.get("/section/%s" % section.id)
        self.assertEqual(resp.status_code, 200)

    def test_add_section_page(self):
        resp = self.client.get("/section/add")
        self.assertEqual(resp.status_code, 200)



# Django settings for example_project project.

from __future__ import absolute_import

import os
import sys
from django.conf import global_settings

from .paths import *  # noqa

DEBUG = True
TEMPLATE_DEBUG = DEBUG
try:
    import debug_toolbar  # noqa
    DEBUG_TOOLBAR = True
except:
    DEBUG_TOOLBAR = False

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'sayit-example-project',
        'USER': '',
        'PASSWORD': '',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}

ALLOWED_HOSTS = []

SECRET_KEY = 'secret'

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = os.path.join(PARENT_DIR, 'uploads')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# All uploaded files world-readable
FILE_UPLOAD_PERMISSIONS = 420  # 644 in octal, 'rw-r--r--'

# List of callables that know how to import templates from various sources.
loaders = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    # 'django.template.loaders.eggs.Loader',
)
if not DEBUG:
    loaders = (('django.template.loaders.cached.Loader', loaders),)

TEMPLATE_LOADERS = loaders

MIDDLEWARE_CLASSES = [
    'django.middleware.gzip.GZipMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'speeches.middleware.InstanceMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
]
if DEBUG_TOOLBAR:
    MIDDLEWARE_CLASSES.append('debug_toolbar.middleware.DebugToolbarMiddleware')

INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'example_project.urls'

LOGIN_REDIRECT_URL = '/'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'example_project.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(PROJECT_DIR, 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    tuple(global_settings.TEMPLATE_CONTEXT_PROCESSORS) + ("django.core.context_processors.request",))

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'django.contrib.admin',
    'django.contrib.admindocs',
    'haystack',
    'django_select2',
    'django_bleach',
    'popolo',
    'instances',
    'speeches',
    'easy_thumbnails',
]

try:
    import tastypie  # noqa
    INSTALLED_APPS.append('tastypie')
except ImportError:
    pass

try:
    import nose  # noqa
    INSTALLED_APPS.append('django_nose')
    TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
    NOSE_PLUGINS = ['speeches.tests.nose_plugins.SkipMigrations']
except ImportError:
    # This is only present in 1.6+, but you shouldn't be running the tests
    # without installing the test requirements anyway
    TEST_RUNNER = 'django.test.runner.DiscoverRunner'

if DEBUG_TOOLBAR:
    INSTALLED_APPS.append('debug_toolbar')

# Log WARN and above to stderr; ERROR and above by email when DEBUG is False.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins', 'console'],
            'level': 'WARN',
            'propagate': True,
        },
        'speeches': {
            'handlers': ['mail_admins', 'console'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'pyelasticsearch': {
            'handlers': ['console'],
            'level': 'WARN',
            'propagate': True,
        },
        'requests.packages.urllib3.connectionpool': {
            'handlers': ['console'],
            'level': 'WARN',
            'propagate': True,
        },
    }
}

# pagination related settings
PAGINATION_DEFAULT_WINDOW = 2

APPEND_SLASH = False

# Select2
AUTO_RENDER_SELECT2_STATICS = False

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(PARENT_DIR, 'collected_static')

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

if 'test' not in sys.argv:
    STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.CachedStaticFilesStorage'

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# django-bleach configuration
from .bleach import *  # noqa

# easy-thumbnails configuration
from .thumbnails import *  # noqa

# Haystack search settings

SEARCH_INDEX_NAME = DATABASES['default']['NAME']
if 'test' in sys.argv:
    SEARCH_INDEX_NAME += '_test'

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.elasticsearch_backend.ElasticsearchSearchEngine',
        'URL': 'http://127.0.0.1:9200/',
        'INDEX_NAME': SEARCH_INDEX_NAME,
    },
    'write': {
        'ENGINE': 'haystack.backends.elasticsearch_backend.ElasticsearchSearchEngine',
        'URL': 'http://127.0.0.1:9200/',
        'INDEX_NAME': '%s_write' % SEARCH_INDEX_NAME,
    },
}

HAYSTACK_SIGNAL_PROCESSOR = 'haystack.signals.RealtimeSignalProcessor'

# Allow local changes of settings
try:
    with open(SETTINGS_DIR + '/local.py') as f:
        exec(compile(f.read(), 'local.py', 'exec'))
except IOError:
    pass



import logging

logger = logging.getLogger(__name__)


class ImporterBase(object):
    def __init__(self, instance=None, commit=True, clobber=None, verify=True, **kwargs):
        self.instance = instance
        self.commit = commit
        self.clobber = clobber
        self.verify = verify
        self.speakers = {}

        self.stats = {}

    def make(self, cls, **kwargs):
        self.stats.setdefault(cls, 0)
        self.stats[cls] += 1

        s = cls(instance=self.instance, **kwargs)
        if self.commit:
            s.save()
        elif s.heading:
            logger.info(s.heading)
        return s



from django.db.models import Aggregate
from django.db.models.sql.aggregates import Aggregate as SQLAggregate


class SQLLength(SQLAggregate):
    sql_function = 'LENGTH'


class Length(Aggregate):
    name = 'Length'

    def add_to_query(self, query, alias, col, source, is_summary):
        aggregate = SQLLength(col, source=source, is_summary=is_summary, **self.extra)
        query.aggregates[alias] = aggregate
